using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Tools;


namespace PopupTest
{
  public class TextBoxWithPopup : TextBox, IPopupParent
  {
    private bool _disposed;
    private PopupControlContainer popupContainer;
    private TextBox popupTextBox_;
    private string _stateData = string.Empty;
    public Control friendlyControl_;
    public bool HandleDownKey = false;
    
    public PopupControlContainer PopupControlContainer
    {
      get
      {
        return popupContainer;
      }
      set
      {
        if (value != null)
        {
          if(popupContainer != value)
          {
            popupContainer = value;
            popupContainer.PopupParent = this;            
            // Make sure to unsubscribe somewhere in your application.
          }
        }
        else
        {
          popupContainer = null;  
        }
      }
    }
   
    // The text box in the popup.
    public TextBox PopupTextBox
    {
      get{return popupTextBox_;}
      set{popupTextBox_ = value;}
    }

    // Control which when selected will not close the popup.
    public Control FriendlyControl
    {
      get{return friendlyControl_;}
      set{friendlyControl_ = value;}
    }


    protected override void Dispose( bool disposing )
    {
      if(!_disposed && disposing)
      {
        _disposed = true;
      }
      base.Dispose(disposing);
    }

    protected override void OnLostFocus(EventArgs args_)
    {
      if (PopupControlContainer != null && PopupControlContainer.IsShowing())
      {
        PopupControlContainer.HidePopup(PopupCloseType.Done);
      }
      base.OnLostFocus(args_);
    }
    // Open/Close popup.
    protected override void OnKeyDown(KeyEventArgs e)
    {
      if (PopupControlContainer != null)
      {
        // If user pressed key down, then show the popup.
        if(HandleDownKey && e.Alt && e.KeyCode == Keys.Down
          && !PopupControlContainer.IsShowing())
        {
          // Let the popup align around the source textBox.
          PopupControlContainer.ParentControl = popupTextBox_;
          // Passing Point.Empty will align it automatically around the above ParentControl.
          PopupControlContainer.ShowPopup(Point.Empty);
          e.Handled = true;
        }
        else
        {
          if(PopupControlContainer.IsShowing())
          {
            if ((e.KeyCode == Keys.Down || e.KeyCode == Keys.Up || e.KeyCode == Keys.Left || e.KeyCode == Keys.Right)){}
            else
            {
              if (e.KeyCode == Keys.Enter)
              {
                PopupControlContainer.HidePopup(PopupCloseType.Done);
              }
              else
              {
                PopupControlContainer.HidePopup(PopupCloseType.Canceled);
              }
              e.Handled = true;
            }
          }
          else
          {
            if (e.KeyCode == Keys.Escape)
            {
              // Clear the text.
              Text = "";
              e.Handled = true;
            }
          }
        }
      }
      base.OnKeyDown(e);
    }
 
		#region IPopupParent_Imp
    public Control GetPopupParentControl()
    {
      // Popup will align around this control if GetLocationForPopupAlignment\
      // returned Point.Empty.
      return this;
    }
    public bool IsRelatedControl(Control control, bool askPopupParent)
    {
      // The popup will then not lose focus automatically when Controls below
      // get the focus.
      if(control == friendlyControl_ || control == this)
        return true;
      else
        return false;
    }
    public void ChildClosing(IPopupChild childUI, PopupCloseType popupCloseType)
    {
      // Transfer data from the popup.
      if(popupCloseType == PopupCloseType.Done)
      {
        Text = PopupTextBox.Text;
      }
      // Set focus back to textbox.
      if(popupCloseType == PopupCloseType.Done
        || popupCloseType == PopupCloseType.Canceled)
        Focus();
    }
    // Lets you provide a preferred location for popup.
    public Point GetLocationForPopupAlignment(PopupRelativeAlignment prevAlignment, 
      out PopupRelativeAlignment newAlignment)
    {
      Rectangle bounds = Bounds;
      // This utility method takes a rectangle and returns the 8 positions around
      // it 1 at a time. It will start with the alignment specified in the 3rd argument.
      Point pos = PopupUtils.ComputeDefaultPopupAlignment(prevAlignment, out newAlignment,
        PopupRelativeAlignment.BottomLeft, PopupRelativeAlignment.BottomLeft, bounds);

      return Parent.PointToScreen(pos);
    }

    public Point[] GetBorderOverlapCue(PopupRelativeAlignment relativeAlignment)
    {
      Rectangle bounds = Bounds;

      // This utility fucntion takes a rectangular bounds and relative alignemnt
      // and returns a line (array of 2 points) that represet the boundary
      // for the specified alignemnt.
      return PopupUtils.ComputeDefaultBorderOverlapCue(relativeAlignment, 
        RectangleToScreen(bounds));
    }
    public bool IsRightToLeft
    {
      get
      {
        return false;
      }
    }
		#endregion IPopupParent_Imp

    #region Protected Methods

    protected override void OnClick(System.EventArgs e)
    {
      if(PopupControlContainer != null && PopupControlContainer.IsShowing())
      {
        PopupControlContainer.HidePopup(PopupCloseType.Canceled);
      }
      base.OnClick(e);
    }
   #endregion Protected Methods

  }
}